﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.DecisionList.Evaluator {
  [Item("MDLIterationOperator", "Description missing")]
  [StorableClass]
  public class MDLIterationOperator : SingleSuccessorOperator {

    #region Parameter Properties
    public ILookupParameter<DecisionList> DecisionListParameter {
      get { return (ILookupParameter<DecisionList>)Parameters["DecisionList"]; }
    }
    public IValueLookupParameter<IDecisionListClassificationProblemData> ProblemDataParameter {
      get { return (IValueLookupParameter<IDecisionListClassificationProblemData>)Parameters["ProblemData"]; }
    }
    public IValueLookupParameter<MDLCalculator> MDLCalculatorParameter {
      get { return (IValueLookupParameter<MDLCalculator>)Parameters["MDLCalculator"]; }
    }
    public IValueLookupParameter<IntValue> IterationParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["Iteration"]; }
    }
    public IValueLookupParameter<IntValue> MDLIterationParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["MDLIteration"]; }
    }
    public IValueLookupParameter<DoubleValue> InitialTheoryLengthRationParameter {
      get { return (IValueLookupParameter<DoubleValue>)Parameters["InitialTheoryLengthRation"]; }
    }
    public IValueLookupParameter<DoubleValue> WeightRelaxFactorParameter {
      get { return (IValueLookupParameter<DoubleValue>)Parameters["WeightRelaxFactor"]; }
    }
    #endregion

    [StorableConstructor]
    protected MDLIterationOperator(bool deserializing) : base(deserializing) { }
    protected MDLIterationOperator(MDLIterationOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public MDLIterationOperator()
      : base() {
      Parameters.Add(new LookupParameter<DecisionList>("DecisionList", ""));
      Parameters.Add(new ValueLookupParameter<IDecisionListClassificationProblemData>("ProblemData", ""));
      Parameters.Add(new ValueLookupParameter<IntValue>("Iteration", ""));
      Parameters.Add(new ValueLookupParameter<IntValue>("MDLIteration", ""));
      Parameters.Add(new ValueLookupParameter<MDLCalculator>("MDLCalculator", ""));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MDLIterationOperator(this, cloner);
    }

    public override IOperation Apply() {
      if (MDLCalculatorParameter.ActualValue == null) {
        MDLCalculatorParameter.ActualValue = new MDLCalculator(MDLIterationParameter.ActualValue.Value,
                                                               InitialTheoryLengthRationParameter.ActualValue.Value,
                                                               WeightRelaxFactorParameter.ActualValue.Value);
      }
      MDLCalculatorParameter.ActualValue.StartNewIteration(new DecisionListSolution(DecisionListParameter.ActualValue, ProblemDataParameter.ActualValue), IterationParameter.ActualValue.Value);
      return base.Apply();
    }
  }
}
